
package hr.webtech.pay2.data;

import android.os.Parcel;
import android.os.Parcelable;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;

import com.google.gson.annotations.Expose;
import com.google.gson.annotations.SerializedName;

import hr.webtech.pay2.util.Objects;
import hr.webtech.pay2.util.Utils;
import hr.webtech.pay2.view.WebtehWebView;


/**
 * PayCvvModel - used in {@link WebtehWebView#payCvv(PayCvvModel)}
 * <p>
 * Check <a href='https://gist.github.com/jasminsuljic/ec63e345dd7a39248f5cdd9d45130529'> json schema</a>
 */
public final class PayCvvModel implements Parcelable {

    /**
     * (Required)
     */
    @SerializedName("merchant_id")
    @Expose
    String merchantId;
    /**
     * short description of order being processed
     */
    @SerializedName("order_info")
    @Expose
    String orderInfo;
    /**
     * unique identifier
     * (Required)
     */
    @SerializedName("order_number")
    @Expose
    String orderNumber;
    @SerializedName("sign")
    @Expose
    String sign;
    /**
     * amount is in minor units, ie. 10.24 USD is sent as 1024
     * (Required)
     */
    @SerializedName("amount")
    @Expose
    String amount;
    /**
     * (Required)
     */
    @SerializedName("currency")
    @Expose
    String currency;
    /**
     * (Required)
     */
    @SerializedName("client_secret")
    @Expose
    String clientSecret;
    /**
     * (Required)
     */
    @SerializedName("auth_token")
    @Expose
    String authToken;
    /**
     * (Required)
     */
    @SerializedName("client")
    @Expose
    String client;
    /**
     * (Required)
     */
    @SerializedName("pm_alias")
    @Expose
    String pmAlias;


    /**
     * (Required)
     */
    @SerializedName("digest")
    @Expose
    String digest;

    @SerializedName("address")
    @Expose
    @Nullable
    String address;

    @SerializedName("phone_number")
    @Expose
    @Nullable
    String phoneNumber;

    @SerializedName("city")
    @Expose
    @Nullable
    String city;

    final static Parcelable.Creator<PayCvvModel> CREATOR = new Creator<PayCvvModel>() {


        @SuppressWarnings({
                "unchecked"
        })
        public PayCvvModel createFromParcel(Parcel in) {
            return new PayCvvModel(in);
        }

        public PayCvvModel[] newArray(int size) {
            return (new PayCvvModel[size]);
        }

    };


    private PayCvvModel(Parcel in) {
        merchantId = in.readString();
        orderInfo = in.readString();
        orderNumber = in.readString();
        sign = in.readString();
        amount = in.readString();
        currency = in.readString();
        clientSecret = in.readString();
        authToken = in.readString();
        pmAlias = in.readString();
        client = in.readString();
        address = in.readString();
        phoneNumber = in.readString();
        city = in.readString();

        generateDigest();
    }

    public PayCvvModel(String merchantId, String orderInfo, String orderNumber, String sign, String amount, String currency, String clientSecret, String authToken, String client, String pmAlias, @Nullable String address, @Nullable String phoneNumber, @Nullable String city) {

        this.merchantId = merchantId;
        this.orderInfo = orderInfo;
        this.orderNumber = orderNumber;
        this.sign = sign;
        this.amount = amount;
        this.currency = currency;
        this.clientSecret = clientSecret;
        this.authToken = authToken;
        this.client = client;
        this.pmAlias = pmAlias;
        this.address = address;
        this.phoneNumber = phoneNumber;
        this.city = city;

        generateDigest();
    }

    public PayCvvModel(@NonNull Merchant merchant, @NonNull PayCvvOrderDetails payCvvOrderDetails, @Nullable Customer customer) {

        Objects.requireNonNull(merchant);
        Objects.requireNonNull(payCvvOrderDetails);

        final OrderDetails orderDetails = payCvvOrderDetails.getOrderDetails();

        this.merchantId = merchant.getMerchantId();
        this.orderInfo = orderDetails.getOrderInfo();
        this.orderNumber = orderDetails.getOrderNumber();
        this.sign = orderDetails.getSignature();
        this.amount = orderDetails.getAmount();
        this.currency = orderDetails.getCurrency();
        this.clientSecret = merchant.getClientSecret();
        this.authToken = merchant.getAuthenticationToken();
        this.client = orderDetails.getClient();
        this.pmAlias = payCvvOrderDetails.getPmAlias();

        if (customer == null) {
            this.address = null;
            this.phoneNumber = null;
            this.city = null;
        } else {
            this.address = customer.getAddress();
            this.phoneNumber = customer.getPhoneNumber();
            this.city = customer.getCity();
        }

        generateDigest();
    }

    private void generateDigest() {
        String sb1 = merchantId +
                pmAlias +
                authToken +
                orderNumber +
                sign +
                amount +
                currency +
                clientSecret;

        this.digest = Utils.sha1Hash(sb1);
    }

    /**
     * @param pmAlias
     * @param amount
     * @param sign
     * @param orderInfo
     * @param client
     * @param orderNumber
     * @param authToken
     * @param merchantId
     * @param clientSecret
     * @param currency
     */
    public PayCvvModel(String merchantId, String orderInfo, String orderNumber, String sign, String amount, String currency, String clientSecret, String authToken, String client, String pmAlias) {
        this(merchantId, orderInfo, orderNumber, sign, amount, currency, clientSecret, authToken, client, pmAlias, null, null, null);
    }

    public void writeToParcel(Parcel dest, int flags) {
        dest.writeValue(merchantId);
        dest.writeValue(orderInfo);
        dest.writeValue(orderNumber);
        dest.writeValue(sign);
        dest.writeValue(amount);
        dest.writeValue(currency);
        dest.writeValue(clientSecret);
        dest.writeValue(authToken);
        dest.writeValue(pmAlias);
        dest.writeString(client);
        dest.writeString(address);
        dest.writeString(phoneNumber);
        dest.writeString(city);
    }

    public String getOrderNumber() {
        return orderNumber;
    }

    public int describeContents() {
        return 0;
    }

}